// Copyright (c) 2018 Shapelets.io
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

#include <khiva_c/defines.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Primitive of the findBestNDiscords function.
 *
 * @param profile The matrix profile containing the minimum distance of each
 * subsequence
 * @param The matrix profile index containing the index of the most similar
 * subsequence
 * @param length_profile Length of the matrix profile
 * @param n Number of discords to extract
 * @param discord_distances The distance of the best N discords
 * @param discord_indices The indices of the best N discords
 * @param subsequence_indices The indices of the query sequences that produced
 * the "N" bigger discords.
 */
KHIVAAPI void find_best_n_discords(khiva_array *profile, khiva_array *index, long *n, khiva_array *discord_distances,
                                   khiva_array *discord_indices, khiva_array *subsequence_indices);

/**
 * @brief Primitive of the findBestNMotifs function.
 *
 * @param profile The matrix profile containing the minimum distance of each
 * subsequence.
 * @param index The matrix profile index containing where each minimum occurs.
 * @param length_profile Length of the matrix profile.
 * @param n Number of motifs to extract.
 * @param motif_distances The distance of the best N motifs.
 * @param motif_indices The indices of the best N motifs.
 * @param subsequence_indices The indices of the query sequences that produced
 * the minimum reported in the motifs.
 */
KHIVAAPI void find_best_n_motifs(khiva_array *profile, khiva_array *index, long *n, khiva_array *motif_distances,
                                 khiva_array *motif_indices, khiva_array *subsequence_indices);

/**
 * @brief  Primitive of the STOMP algorithm.
 *
 * @param tssa Query time series
 * @param tssb Reference time series
 * @param m Pointer to a long with the length of the subsequence.
 * @param p The matrix profile, which reflects the distance to the closer element of the subsequence
 * from 'tssa' in 'tssb'.
 * @param i The matrix profile index, which points to where the aforementioned minimum is located.
 */
KHIVAAPI void stomp(khiva_array *tssa, khiva_array *tssb, long *m, khiva_array *p, khiva_array *i);

/**
 * @brief Primitive of the STOMP self join algorithm.
 *
 * @param tss Query and reference time series
 * @param m Pointer to a long with the length of the subsequence.
 * @param p The matrix profile, which reflects the distance to the closer element of the subsequence
 * from 'tss' in a different location of itself
 * @param i The matrix profile index, which points to where the aforementioned minimum is located
 */
KHIVAAPI void stomp_self_join(khiva_array *tss, long *m, khiva_array *p, khiva_array *i);

#ifdef __cplusplus
}
#endif
